require 'uu/os/application/use_case_context'
require 'uu/os/vuc/view_model'
require 'uu/os/vuc/action'

module UU
  module OS
    module VUC
      # Visual use case used in controller implementation.
      class VisualUseCaseContext < UU::OS::Application::UseCaseContext

        # Creates new instance of execution context.
        # @param [Hash] data Initialization data:
        # @option data [URI, String] :uu_uri URI of main object.
        # @option data [Hash] :parameters Use case parameters.
        # @option data [UU::OS::Security::Session] :session Instance of session object.
        # @option data [UU::OS::HTTP::HTTPRequest] :request Instance of request.
        # it means that invocation of invoke_async and invoke_async_status ends up with exception.
        # @private
        def initialize(data = {}) # :nodoc:
          super(data)
        end

        # Returns object which is used for working with components and building common response.
        # You can use this object if you want:
        #   * work with components, get value, change component attribute etc.
        #   * add user message
        #   * change focused component
        #   * change view template
        #   * get component, which trigger action
        #   * add custom attribute to be stored within actions
        # @return [UU::OS::VUC::ViewModel] Object which is serialized to View response message.
        # @see #UU::OS::VUC::ViewModel
        def view
          @view_model ||= UU::OS::VUC::ViewModel.new(self)
        end


        # Returns object which is used for define action, action can be:
        #   * back - Returns to previous Use case
        #   * super uc - Execute super Use case instead of this one
        #   * forward - Navigate to another Use case
        #   * dialog - Open modal dialog
        #   * progress dialog - Open progress dialog
        #   * download - Creates action for launching download from client (browser).
        #
        # @return [UU::OS::VUC::Action] Object which is serialized to Action response message.
        # @see #UU::OS::VUC::Action
        def action
          @action ||= UU::OS::VUC::Action.new(self)
        end

      end
    end
  end
end
